class ResyncRequest(object):
    def __init__(self, period=None, event_types=None, last_row_id=-1):
        self.period = period
        self.event_types = event_types
        self.last_row_id = last_row_id
