class Table(object):
    def __init__(
            self,                       # type: str
            id="",                      # type: str
            name="",                    # type: str
            current_pos_id="",          # type: str
            status="",                  # type: str
            tab_id="",                  # type: str
            business_period="",         # type: str
            sector="",                  # type: str
            slice_parent_table_id="",   # type: str
            slice_source_table_id="",   # type: str
            mode=""                     # type: str
    ):
        # type: (...) -> None

        self.id = id
        self.name = name
        self.current_pos_id = current_pos_id
        self.status = status
        self.tab_id = tab_id
        self.business_period = business_period
        self.sector = sector
        self.slice_source_table_id = slice_source_table_id
        self.slice_parent_table_id = slice_parent_table_id
        self.mode = mode
