class OrderInsertPaymentResponse(object):
    def __init__(self, tender):
        # type: (dict) -> None

        self.dueAmount = tender.get("dueAmount")
        self.tenderId = tender.get("tenderId")
