from abc import ABCMeta, abstractmethod
from xml.etree import cElementTree as eTree

from model.order_detail import OrderDetail


class OrderServiceModel(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def convert_mw_order_to_order_detail(self, order):
        # type: (eTree) -> OrderDetail

        raise NotImplementedError
