from abc import ABCMeta, abstractmethod


class PaymentResponse(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def to_xml(self):
        # type: () -> str
        raise NotImplementedError()
