class InvalidTableEventException(RuntimeError):
    def __init__(self, data):
        message = 'Invalid Table Event. Event: "{}"'.format(data)
        super(InvalidTableEventException, self).__init__(message)
