from httpmessageprocessorutil.automapped import auto_map
from typing import List


@auto_map({"system_api_key": {"name": "X-Api-Key", "location": "header", "type": str},
           "pos_id": {"name": "posId", "type": str},
           "line_number": {"name": "lineNumber", "type": List[int]}})
class OrderDeleteLineRequest(object):
    def __init__(self, system_api_key, pos_id, line_numbers):
        # type: (str, str, List[int]) -> None

        self.system_api_key = system_api_key
        self.pos_id = pos_id
        self.line_numbers = line_numbers
