from model.electronic_message import ElectronicMessage
from typing import List


class ReadMessagesResponse(object):
    def __init__(self, messages):
        # type: (List[ElectronicMessage]) -> None

        self.messages = messages
