from httpmessageprocessorutil.automapped import auto_map


@auto_map({"system_api_key": {"name": "X-Api-Key", "location": "header", "type": str}})
class GetRupturedProductsRequest(object):
    def __init__(self, system_api_key):
        # type: (str) -> None

        self.system_api_key = system_api_key
