from enum import Enum


class ActionTypes(Enum):
    itemsToAdd = "itemsToAdd"
    discountPercentageOnTotal = "discountPercentageOnTotal"
    discountPercentageByItems = "discountPercentageByItems"
    discountPercentageByGroupsOnTotal = "discountPercentageByGroupsOnTotal"
    discountAmountByGroupsOnTotal = "discountAmountByGroupsOnTotal"
    discountAmountOnTotal = "discountAmountOnTotal"
    discountAmountByItems = "discountAmountByItems"
    paymentAmount = "paymentAmount"
    paymentPercentage = "paymentPercentage"
    paymentAmountOnItem = "paymentAmountOnItem"
    discountPercentageByItemsOnTotal = "discountPercentageByItemsOnTotal"
    discountAmountByItemsOnTotal = "discountAmountByItemsOnTotal"
