from application.repository import AccountRepository
from systools import sys_log_exception

from .. import mb_context

account_repository = AccountRepository(mb_context)


def get_drawer_initial_amount(pos_id, session):
    try:
        return account_repository.get_initial_amount(pos_id, session)
    except BaseException as _:
        sys_log_exception("Error in consult initial amount")
